(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common'], factory) :
	(factory((global['chrome-file-system-api'] = {}),global.core,global.common));
}(this, (function (exports,core,common) { 'use strict';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var PERSISTENT = 1;
var FILE_SYSTEM_SIZE = 1048576 * 1024; // 5MB
var TIMEOUT_FOR_IO = 1000; // 1 second
var STATUS_INITIAZING = 'INITIAZING';
var STATUS_REQUESTED_PERMISSION_WITH_ERROR = 'SRPWE';


var STATUS_OK = 'OK';
var RETURN_TYPE = {
    BLOB: 'blob',
    TEXT: 'text'
};

var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ChromeFileSystemService = (function () {
    function ChromeFileSystemService() {
        var _this = this;
        this.storage = {
            quota: 0,
            usage: 0
        };
        this.status = STATUS_INITIAZING;
        this.requestQuota().then(function () {
            _this.requestFileSystem().then(function () {
                _this.updateUsageAndQuota().then(function () {
                    _this.status = STATUS_OK;
                });
            });
        });
    }
    /**
     * @param {?} fileName
     * @return {?}
     */
    ChromeFileSystemService.prototype.createFile = /**
     * @param {?} fileName
     * @return {?}
     */
    function (fileName) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.status === STATUS_OK) {
                _this.fs.root.getFile(fileName, { create: true, exclusive: true }, function (fileEntry) {
                    resolve(fileEntry);
                }, function (error) {
                    reject(error);
                });
            }
            else {
                setTimeout(function () {
                    _this.createFile(fileName);
                }, TIMEOUT_FOR_IO);
            }
        });
    };
    /**
     * @param {?} directoryName
     * @return {?}
     */
    ChromeFileSystemService.prototype.createDirectory = /**
     * @param {?} directoryName
     * @return {?}
     */
    function (directoryName) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.status === STATUS_OK) {
                _this.fs.root.getDirectory(directoryName, { create: true }, function (directoryEntry) {
                    console.debug('Directory created:', directoryEntry);
                    resolve(directoryEntry);
                }, function (error) {
                    reject(error);
                });
            }
            else {
                setTimeout(function () {
                    _this.createDirectory(directoryName);
                }, TIMEOUT_FOR_IO);
            }
        });
    };
    /**
     * @param {?} path
     * @param {?} file
     * @return {?}
     */
    ChromeFileSystemService.prototype.writeFile = /**
     * @param {?} path
     * @param {?} file
     * @return {?}
     */
    function (path, file) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.status === STATUS_OK) {
                _this.fs.root.getFile(path, { create: true }, function (fileEntry) {
                    fileEntry.createWriter(function (fileWriter) {
                        fileWriter.onwriteend = function (e) {
                            resolve(e);
                        };
                        fileWriter.onerror = function (e) {
                            reject(e);
                        };
                        fileWriter.write(file);
                    }, function (error) {
                        reject(error);
                    });
                }, function (error) {
                    reject(error);
                });
            }
            else {
                setTimeout(function () {
                    _this.writeFile(path, file);
                }, TIMEOUT_FOR_IO);
            }
        });
    };
    /**
     * @param {?} path
     * @param {?} returnType
     * @return {?}
     */
    ChromeFileSystemService.prototype.readFile = /**
     * @param {?} path
     * @param {?} returnType
     * @return {?}
     */
    function (path, returnType) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.status === STATUS_OK) {
                _this.fs.root.getFile(path, {}, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var /** @type {?} */ reader = new FileReader();
                        if (returnType === RETURN_TYPE.TEXT) {
                            if (file) {
                                reader.readAsText(file);
                            }
                            reader.onerror = function (event) {
                                reject(event);
                            };
                            reader.onloadend = function () {
                                resolve(reader.result);
                            };
                        }
                        else if (returnType === RETURN_TYPE.BLOB) {
                            resolve(file);
                        }
                    });
                });
            }
            else {
                setTimeout(function () {
                    _this.readFile(path, returnType);
                }, TIMEOUT_FOR_IO);
            }
        });
    };
    /**
     * @param {?} fileName
     * @return {?}
     */
    ChromeFileSystemService.prototype.deleteFile = /**
     * @param {?} fileName
     * @return {?}
     */
    function (fileName) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.status === STATUS_OK) {
                _this.fs.root.getFile(fileName, { create: false }, function (fileEntry) {
                    fileEntry.remove(function (ev) { return __awaiter(_this, void 0, void 0, function () {
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, this.updateUsageAndQuota()];
                                case 1:
                                    _a.sent();
                                    resolve(ev);
                                    return [2 /*return*/];
                            }
                        });
                    }); }, function (error) {
                        reject(error);
                    });
                }, function (error) { return reject(error); });
            }
            else {
                setTimeout(function () {
                    _this.deleteFile(fileName);
                }, TIMEOUT_FOR_IO);
            }
        });
    };
    /**
     * @param {?} directoryName
     * @return {?}
     */
    ChromeFileSystemService.prototype.deleteDirectory = /**
     * @param {?} directoryName
     * @return {?}
     */
    function (directoryName) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.status === STATUS_OK) {
                _this.fs.root.getDirectory(directoryName, {}, function (directoryEntry) {
                    resolve(directoryEntry.removeRecursively(function () { return console.debug('Directory removed:', directoryName); }));
                }, function (error) {
                    reject(error);
                });
            }
            else {
                setTimeout(function () {
                    _this.deleteDirectory(directoryName);
                }, TIMEOUT_FOR_IO);
            }
        });
    };
    /**
     * @return {?}
     */
    ChromeFileSystemService.prototype.requestQuota = /**
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            (/** @type {?} */ (navigator)).webkitPersistentStorage.requestQuota(FILE_SYSTEM_SIZE, function (grantedBytes) {
                _this.storage['quota'] = grantedBytes;
                _this.status = STATUS_OK;
                resolve(grantedBytes);
            }, function (error) {
                _this.status = STATUS_REQUESTED_PERMISSION_WITH_ERROR;
                reject(error);
            });
        });
    };
    /**
     * @return {?}
     */
    ChromeFileSystemService.prototype.updateUsageAndQuota = /**
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            (/** @type {?} */ (navigator)).webkitPersistentStorage.queryUsageAndQuota(function (usedBytes, grantedBytes) {
                _this.storage['quota'] = grantedBytes;
                _this.storage['usage'] = usedBytes;
                resolve();
            }, function (error) {
                reject(error);
            });
        });
    };
    /**
     * @return {?}
     */
    ChromeFileSystemService.prototype.requestFileSystem = /**
     * @return {?}
     */
    function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            (/** @type {?} */ (window)).webkitRequestFileSystem(PERSISTENT, FILE_SYSTEM_SIZE, function (fs) {
                _this.fs = fs;
                resolve();
            }, function (error) {
                reject(error);
            });
        });
    };
    /**
     * @return {?}
     */
    ChromeFileSystemService.prototype.waitForIO = /**
     * @return {?}
     */
    function () {
    };
    ChromeFileSystemService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    ChromeFileSystemService.ctorParameters = function () { return []; };
    return ChromeFileSystemService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ChromeFileSystemModule = (function () {
    function ChromeFileSystemModule() {
    }
    /**
     * @return {?}
     */
    ChromeFileSystemModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: ChromeFileSystemModule,
            providers: [ChromeFileSystemService]
        };
    };
    ChromeFileSystemModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [],
                    exports: []
                },] },
    ];
    return ChromeFileSystemModule;
}());

exports.ChromeFileSystemModule = ChromeFileSystemModule;
exports.ChromeFileSystemService = ChromeFileSystemService;

Object.defineProperty(exports, '__esModule', { value: true });

})));
